/*
 *  Jomar3D, a lobster-running-to-catch game.
 *  Copyright (C) 2006-2008 Jonathan Michalon <studios.chalmion@no-log.org>.
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#ifndef JOMAR3D_H
#  define JOMAR3D_H


#  ifndef DEBUG
#    define DEBUG 0             /* 0: false, no debug   1: first level   2: a very long poem every step */
#  endif

#  ifndef USEGLEW
#    define USEGLEW 0           /* whether we use glew, needed to use OpenGL extensions */
#  endif

#  define MYFREEGLUT 0          /* NOT WORKING whether we have freeglut or the basic glut (interesting options) */

/* Default directory definition : when not given by make */
#  ifndef DATADIR
#    define DATADIR "."
#  endif


/* number of martians */
#  define nmartians 10
#  if (DEBUG)
#    define nmartians 1
#  endif
int martleft;

/* field of view of the Martians (squares) */
#  define martview 3

/* size of the ground (steps) */
#  define T_X 40
#  define T_Y 30
/* size of each square of the ground (OpenGL units) */
#  define T_C 70.
/* height of the wells */
#  define T_H 80.

#  define MAXCMD 1024


int g_winmode;
int winwidth, winheight;
int winuserdef;
float ratio;                    // ratio of the screen, 0 uninitialized (useful to know when the window has been created)
float anglehom, anglebasehom, rotdemhom, boostrac;
int gamestarttime;              // time at which the game started

int martact[nmartians];         // action of each martian: 0: idle (looking around), 1: moving
int martposx[nmartians];        // position on x of each martian
int martposy[nmartians];        // position on y of each martian
float martposxgo[nmartians];    // x position of each martian from its goal
float martposygo[nmartians];    // y position of each martian from its goal
float martangle[nmartians];     // angle in degrees of each martian
float martgoangle[nmartians];   // goal angle in degrees of each martian
float martspeed[nmartians];     // time that takes each martian to move one case (x1000) ; 0 if eatten
float marttime[nmartians];      // time at which each martian had started to move
long unsigned int martsteps;

char *music;                    // name of the current music (to be shown)

/*
struct MartPos {
   int x;
   int y;
}

static MartPos[nmartians] *martpos;

int get_martpos (int martian_n, MartPos* pos)
{
   if (martian_n > nmartians || martian_n < 0) return -1;
   
   pos->x = martpos[martian_n]->x;
   pos->y = martpos[martian_n]->y;
   
   return 1;
}
*/

float gone, godist;
int action;

int terrain[T_X][T_Y];          // = 0      : empty

                              // = T_H    : wall
                              // = 1-4    : lobster : 1: >y+      2: >x+       3: >y-       4: >x-
                              // = 11-14  : martian : 11: >y+       12: >x+       13: >y-       14: >x-


enum
{
  T_EMPTY = 0,                  //< Empty
  T_LOBSTER_FORWARD = 1,        //< y+
  T_LOBSTER_RIGHT,              //< x+
  T_LOBSTER_BACKWARD,           //< y-
  T_LOBSTER_LEFT,               //< x-
  T_MARTIAN_FORWARD = 11,       //< y+
  T_MARTIAN_RIGHT,              //< x+
  T_MARTIAN_BACKWARD,           //< y-
  T_MARTIAN_LEFT,               //< x-
  T_WALL,                       //< Wall
  T_LOCKED                      //< Position locked (init)
};

#  define terrain_check(x,y,v) ((terrain[x][y] == v) ? 1 : 0)
#  define terrain_lobster(x,y) \
   ((terrain_check (x,y,T_LOBSTER_BACKWARD)  || \
     terrain_check (x,y,T_LOBSTER_FORWARD)   || \
     terrain_check (x,y,T_LOBSTER_LEFT)      || \
     terrain_check (x,y,T_LOBSTER_RIGHT)) ? 1 : 0)
#  define terrain_martian(x,y) \
   ((terrain_check (x,y,T_MARTIAN_BACKWARD)  || \
     terrain_check (x,y,T_MARTIAN_FORWARD)   || \
     terrain_check (x,y,T_MARTIAN_LEFT)      || \
     terrain_check (x,y,T_MARTIAN_RIGHT)) ? 1 : 0)
    // lobster move: 1 forward   2 right   3 left   4 backward

#endif /* guard */
